import 'package:dash_kit_network/src/models/token_pair.dart';
import 'package:dash_kit_network/src/refresh_tokens_delegate.dart';
import 'package:dash_kit_network/src/token_storage.dart';
import 'package:dio/dio.dart';

/// RefreshTokensDelegate that includes base implementation for
/// token refreshing operations. Override the implementations that
/// does not fit in your API requirements
abstract class BaseRefreshTokensDelegate extends RefreshTokensDelegate {
  BaseRefreshTokensDelegate(this.tokenStorage);

  final TokenStorage tokenStorage;

  /// Request for tokens refreshing
  @override
  Future<TokenPair> refreshTokens(Dio dio, TokenPair tokenPair);

  /// This method will be called when API cannot update token pair
  /// because of the refresh token is expired.
  /// User should be redirected to sign-in screen in that case
  @override
  void onTokensRefreshingFailed();

  @override
  Future<TokenPair> getTokens() async {
    final accessToken = await tokenStorage.getAccessToken() ?? '';
    final refreshToken = await tokenStorage.getRefreshToken() ?? '';

    return TokenPair(
      accessToken: accessToken,
      refreshToken: refreshToken,
    );
  }

  @override
  Future<void> updateTokens(TokenPair tokenPair) {
    return tokenStorage.saveTokens(
      accessToken: tokenPair.accessToken,
      refreshToken: tokenPair.refreshToken,
    );
  }

  @override
  bool isAccessTokenExpired(DioError error) {
    return error.response?.statusCode == 401;
  }

  @override
  bool isRefreshTokenExpired(DioError error) {
    return error.response?.statusCode == 401;
  }

  @override
  Options appendAccessTokenToRequest(
    Options options,
    TokenPair? tokenPair,
  ) {
    if (tokenPair == null) {
      return options;
    }

    options.headers?['Authorization'] = 'Bearer ${tokenPair.accessToken}';
    return options;
  }
}
